 PROGRAM implizite_Funktion_2;

 (* Es soll eine implizit gegebene Funktion gezeichnet werden:             *)
 (* gegebene Funktion ist z.B: X^4 + Y^4 = 50                              *)
 (* Unterschied zur Version1 ist:                                          *)
 (* Es werden zwei Durchgaenge gerechnet. Beim ersten Durchgang werden     *)
 (* fuer zwei benachbarte, nur durch die Schrittweite unterschiedliche     *)
 (* Werte XA und XB der dazugehoerige Funktionswert ZA und ZB berechnet    *)
 (* Der Wert fuer Y ist ja in der inneren Schleife konstant und wird in    *)
 (* der aeusseren Schleife schrittweise erhoeht. Beim zweiten Durchgang    *)
 (* wird ZA und ZB fuer zwei benachbarte Y-Werte YA und YB berechnet.      *)
 (* Ob ein Punkt in der Epsilon Umgebung liegt, wird nicht direkt durch    *)
 (* eine Wertabfrage des Absolutbetrages von Z bewerkstelligt, sondern     *)
 (* durch eine Diskussion der Vorzeichen zweier benachbarter Werte ZA,ZB   *)
 (* entschieden; mittels der Signumfunktion, welche als Unterprogramm      *)
 (* implementiert ist.                                                     *)
 (* Zwei Durchgaenge sind notwendig, um das Punktenetz dichter zu          *)
 (* legen. Nach dem 1.ten Durchgang bleiben Luecken bei den Werten, bei    *)
 (* denen die Funktion parallel zur X-Achse liegt. Beim 2.ten Durchgang    *)
 (* bleiben Luecken, wo die Funktion parallel zur Y-Achse verlaeuft        *)
 (* Dieser Bereich wurde jedoch schon sehr genau im ersten Durchgang       *)
 (* abgedeckt. Durchgaenge in Kommentarklammern setzen und ausprobieren!   *)
 (* Es wird ja nicht wie bei der Parameterdarstellung in gleichen          *)
 (* Abstaenden gepunktet.                                                  *)

 USES CRT,GRAPH,WGRAPHIC;
  CONST
  EPS = 1.0E-2;
  VAR
  I,J : INTEGER;
  X,XA,XB,X0,Y,YA,YB,Y0,Z,ZA,ZB,Z0,Xun,Xob,Yun,Yob,DX,DY: REAL;

(* ------------------------ Unterprogramme ------------------------------- *)

 FUNCTION Sgn(A : REAL) : INTEGER; 
  BEGIN
  IF(ABS(A) < EPS) THEN BEGIN
      Sgn := 0;
  END ELSE BEGIN IF (A/ABS(A)) > 0 THEN BEGIN
             Sgn := 1;
           END ELSE BEGIN Sgn := -1;
               END;
           END;      
  END;

(* -------------------------- Hauptprogramm ------------------------------ *)
BEGIN
  WINITGRAPH(0,0,'');                          (* !!!! hier aendern !!!!   *)
  CLEARDEVICE;
  SetBkColor(15);
  SetColor(1);
  Xun := -5.0; Xob := 5.0;
  Yun := -5.0; Yob := 5.0;
  WORLDWINDOW(Xun,Xob,Yun,Yob);
  WHor(0.0,Xun,Xob); Whor(Xob,Xun,Xob); Whor(Xun,Xun,Xob);
  WVer(0.0,Yun,Yob); WVer(Xun,Yun,Yob); WVer(Xob,Yun,Yob);
  (*  Weltfenster gezeichnet   *)


  DX := 0.04; DY := 0.04;              (* Schrittweite waehlen             *)
  (* --------------------- 1.ter Durchgang beginnt ----------------------  *)
  Y := Yun;
  WHILE (Y < Yob ) DO BEGIN     (* aeussere Schleife y-Wert laeuft langsam *)
     XA := Xun; XB := XA + DX;  
     WHILE (XB < Xob) DO BEGIN  (* innere Schleife: Die X-Werte laufen     *)
                                (* schneller als y. Es wird also von unten *)
                                (* nach oben am Schirm gezeichnet          *)
         (* Die Z-Funktion wird fuer je zwei benachbarte X-Werte berechnet *)
         (* der Y-Wert bleibt ja in der inneren Schleife konstant          *)
        
         (* =============================== *)
         ZA := XA*XA*XA*XA + Y*Y*Y*Y - 50;
         ZB := XB*XB*XB*XB + Y*Y*Y*Y - 50;
         (* =============================== *)

         (* Vorzeichendiskussion                                           *)
         (* Jetzt wird die Signum Funktion fuer ZA und ZB aufgerufen       *)
         (* Sie liefert die Werte -1 , 0 oder +1 zurueck                   *)
         (* 3 hoch 2 = 9 Faelle insgesamt                                  *)
         (* 1.Fall: Wenn beide Vorzeichen -1 sind, wird nicht gezeichnet   *)
         (* 2.Fall: Wenn beide Vorzeichen +1 sind, wird nicht gezeichnet   *)
         (* Bei allen anderen Faellen wird ein Punkt gezeichnet            *)
            
         (* 3.und 4.Fall: Vorzeichen sind verschieden, aber nicht Null !   *)
         (*               a: ZA=-1,ZB=+1   b: ZA=+1,ZB=-1                  *)
         (* aehnliche Dreiecke betrachten , Verhaeltnisse aufstellen       *)
         (* (XB-X0)/ZB = (X0-XA)/(-ZA)  lineare Interpolation              *)
         IF Sgn(ZA) * Sgn(ZB) = -1 THEN BEGIN
            X0 := (XA*ZB - XB*ZA)/(ZB-ZA);
            WPixel(X0,Y,5);
         END;

         (* 5.Fall beide Vorzeichen sind 0                                 *)
         (* einfach arithmetischer Mittelwert                              *)
         IF (Sgn(ZA)=0) AND (Sgn(ZB)=0) THEN BEGIN
            X0 := (XA + XB)/2;
            WPixel(X0,Y,5);
         END;

         (* 6.und 7.Fall:       a: ZA=0,ZB=-1        b:ZA=0,ZB=+1          *)
         IF (Sgn(ZA)=0) AND (Sgn(ZB)<>0) THEN BEGIN
            X0 := XA;
            WPixel(X0,Y,5);
         END;

         (* 8.und 9.Fall   a: ZA=-1,ZB=0        b:ZA=+1,ZB=0               *)
         IF (Sgn(ZA)<>0) AND (Sgn(ZB)=0) THEN BEGIN
            X0 := XB;
            WPixel(X0,Y,5);
         END;
         XA := XB;
         XB := XA + DX;
    END;
    Y := Y + DY;
  END;
  (* ----------------  2.ter Durchgang beginnt --------------------------- *)
  X := Xun;
  WHILE (X < Xob ) DO BEGIN
     YA := Yun; YB := YA + DY;
     WHILE (YB < Yob) DO BEGIN
         (* =============================== *)
         ZA := YA*YA*YA*YA + X*X*X*X - 50;
         ZB := YB*YB*YB*YB + X*X*X*X - 50;
         (* =============================== *)
         IF Sgn(ZA) * Sgn(ZB) = -1 THEN BEGIN
            Y0 := (YA*ZB - YB*ZA)/(ZB-ZA);
            WPixel(X,Y0,5);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)=0) THEN BEGIN
            Y0 := (YA + YB)/2;
            WPixel(X,Y0,5);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)<>0) THEN BEGIN
            Y0 := YA;
            WPixel(X,Y0,5);
         END;
         IF (Sgn(ZA)<>0) AND (Sgn(ZB)=0) THEN BEGIN
            Y0 := YB;
            WPixel(X,Y0,5);
         END;
         YA := YB;
         YB := YA + DY;
    END;
    X := X + DX;
  END;
 WClosegraph;
END.
